#include "xparameters.h"
#include "mb_interface.h"
#include "stdio.h"
#include "xutil.h"
#include "xtmrctr_l.h"
#include "xil_macroback.h"
#include "xintc_l.h"
#include "xuartlite_l.h"
#include "xbasic_types.h"
#include "shell.h"
#include "xil_macroback.h"
//#include "microblaze_exceptions_i.h"

volatile unsigned int tick0, tick1, tick2;

int help();
int mem_write(int , char **);
int mem_read(int , char **);
int led_write(int, char **);
void do_debug();

void *last_dump_add;
unsigned int str_to_uint32(char *, char **, int); 
void npi_cmd(int , int , int , int , int );

typedef struct {
	char *CmdStr;
	int (*func)(int argc, char **argv);
} TCommand;

TCommand Cmds[] = { 
	{"HELP" , help            },
	{"?",    help            },
	{"MWR"  , mem_write       },
	{"MRD"  , mem_read        },
	{"DEBU"  , do_debug},
	{ NULL ,    NULL}
};


int help()
{
	//xil_printf("\n\r Add ");
	return 0;
}


int debug_on = 0;

void do_debug()
{
	debug_on = 1 - debug_on;
}

int mem_write(int argc, char **argv)
{

	unsigned int data;
	int *add;

	if(argc < 3) {
		xil_printf( "Argument Count Error!\n");
		return -1; 
	}

	if(*argv[3] == 'h' || *argv[3] == 'H') {
		add = (short int*)str_to_uint32(argv[1], NULL, 0);
		data = (short int)str_to_uint32(argv[2], NULL, 0);
		*(short int*)add = (short int)data;

	}
	else if(*argv[3] == 'b' || *argv[3] == 'B') {
		add = (char *)str_to_uint32(argv[1], NULL, 0);
		data = (char)str_to_uint32(argv[2], NULL, 0);
		*(char *)add = (char)data;
	}
	else {
		add = (int *)str_to_uint32(argv[1], NULL, 0);
		data = str_to_uint32(argv[2], NULL, 0);
		*add = data;
	}

	return 0;

}

void dump_memory(unsigned int *address, int size)
{
	volatile Xuint32 *add, data[8];
	Xuint32 i, j;
	Xuint8 ch;

	add = address;

	for(i = 0; i < size; i += 4) {
		xil_printf( "%08X:",  add);		// display address
		for(j = 0; j < 4; j++) {
			data[j] = *add;
			xil_printf( "%08x", data[j]);	// display data
			xil_printf( " " );				// space
			add++;
		}
		xil_printf( "  " );

		for(j = 0; j < 16; j++) {	// display char
			ch = (data[j] & 0xff00) >> 8;
			if(ch < ' ' || ch > '~') 
				xil_printf( "." );
			else  
				xil_printf("%c", ch);

			ch = data[j] & 0xff;
			if(ch < ' ' || ch > '~') 
				xil_printf( "." );
			else  
				xil_printf("%c", ch);
		}
		xil_printf("\n\r");
	}
}


int mem_read(int argc, char **argv)
{
	int size;

	switch(argc)   {
		case  1 : // command only
			dump_memory((short int *)last_dump_add, 16); 
			//xil_printf("%s", argv[0]);
			break;
		case  2 : // command & only start address
			last_dump_add = (short int *)str_to_uint32(argv[1], NULL, 0);
			dump_memory(last_dump_add, 16); 
			//			xil_printf("%s", argv[0]);
			//			xil_printf("%s", argv[1]);
			break;
		case  3 : // command, start address and count
			last_dump_add = (int *)str_to_uint32(argv[1], NULL, 0);
			size = str_to_uint32(argv[2], NULL, 0);

			if( size < 0 ) 
				size *= -1;
			dump_memory(last_dump_add, size);   
			break;
		default : xil_printf("Argument Count Error!\n");
				  return -1;
	} 

	return 0;
}

void timer_handler(void *baseaddr_p)
{
	int baseaddr = (int)baseaddr_p;
	unsigned int csr;
	csr = XTmrCtr_mGetControlStatusReg(XPAR_AXI_TIMER_0_BASEADDR, 0);
	tick0++;
	tick1++;
	tick2++;
	if(debug_on)
		outbyte('z');
	XTmrCtr_mSetControlStatusReg(XPAR_AXI_TIMER_0_BASEADDR, 0, csr);
}


int init_ext_timer_interrupt()
{
	XIntc_mMasterEnable(XPAR_INTC_SINGLE_BASEADDR);
	XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR ,
			XPAR_AXI_TIMER_0_INTERRUPT_MASK);


	XIntc_RegisterHandler(XPAR_INTC_SINGLE_BASEADDR,
			XPAR_MICROBLAZE_0_INTC_AXI_TIMER_0_INTERRUPT_INTR,
			timer_handler,
			XPAR_AXI_TIMER_0_BASEADDR);

	XTmrCtr_mSetLoadReg(XPAR_AXI_TIMER_0_BASEADDR,
			0,
			7500000/1000);
	XTmrCtr_mSetControlStatusReg(XPAR_AXI_TIMER_0_BASEADDR,
			0,
			XTC_CSR_INT_OCCURED_MASK |
			XTC_CSR_LOAD_MASK );

	XTmrCtr_mSetControlStatusReg(XPAR_AXI_TIMER_0_BASEADDR,
			0,
			XTC_CSR_ENABLE_TMR_MASK |
			XTC_CSR_ENABLE_INT_MASK |
			XTC_CSR_AUTO_RELOAD_MASK |
			XTC_CSR_DOWN_COUNT_MASK);

}

extern char uart_buffer[40];
extern char *uart_s;
extern int uart_cnt;


int main_shell()
{
	int cmdlp;
	int argc;
	char *argv[10];

	int hw_ver;

	//xil_printf("\n\r %s %s\n\r\n\r", __DATE__, __TIME__);

	init_ext_timer_interrupt();
	//init_exception();

	microblaze_enable_interrupts();
	uart_s = &uart_buffer[0];
	uart_cnt = 0;
	xil_printf("xilinx>");
	inbyte();
	while(1) {
		if(uart_gets()) {
			argc = parse_args(uart_buffer, argv);
			if(argc) {
				upper_str(argv[0]);
				cmdlp = 0;
				while(Cmds[cmdlp].CmdStr) {
					if(strcmp(argv[0], Cmds[cmdlp].CmdStr) == 0) {
						xil_printf("\n\r");
						Cmds[cmdlp].func(argc, argv);
						xil_printf("\n\r");
						break;
					}
					cmdlp++;
				}
				if(Cmds[cmdlp].CmdStr == NULL) 
					xil_printf("\n\rInvalid command\n\r");
			}
			else
				xil_printf("\n\r");
			memset(uart_buffer, 0 , sizeof(uart_buffer));
			xil_printf("xilinx> ");
			uart_cnt = 0;
			uart_s = &uart_buffer[0];
		}
	}
}


void main()
{
	main_shell();
}
